@@ -30,8 +30,10 @@
 #include "nautilus-directory.h"
 #include "nautilus-file-attributes.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-icon-factory.h"
 #include "nautilus-file.h"
 #include "nautilus-metadata.h"
+#include "nautilus-program-choosing.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-stock-dialogs.h>
@@ -266,9 +268,7 @@ nautilus_link_desktop_file_local_get_link_type (const char *path)
 static char *
 nautilus_link_desktop_file_get_link_uri_from_desktop (GnomeDesktopItem *desktop_file)
 {
-	char *terminal_command;
 	const char *launch_string;
-	gboolean need_term;
 	const char *type;
 	char *retval;
 
@@ -284,15 +284,9 @@ nautilus_link_desktop_file_get_link_uri_from_desktop (GnomeDesktopItem *desktop_
 		if (launch_string == NULL) {
 			return NULL;
 		}
-
-		need_term = gnome_desktop_item_get_boolean (desktop_file, "Terminal");
-		if (need_term) {
-			terminal_command = eel_gnome_make_terminal_command (launch_string);
-			retval = g_strconcat ("command:", terminal_command, NULL);
-			g_free (terminal_command);
-		} else {
-			retval = g_strconcat ("command:", launch_string, NULL);
-		}
+		
+		launch_string = gnome_desktop_item_get_location (desktop_file);
+		retval = g_strconcat (NAUTILUS_DESKTOP_COMMAND_SPECIFIER, launch_string, NULL);
 	} else if (strcmp (type, "URL") == 0) {
 		/* Some old broken desktop files use this nonstandard feature, we need handle it though */
 		retval = g_strdup (gnome_desktop_item_get_string (desktop_file, "Exec"));
@@ -315,9 +309,9 @@ nautilus_link_desktop_file_get_link_name_from_desktop (GnomeDesktopItem *desktop
 static char *
 nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop_file)
 {
+	GnomeIconLoader *icon_loader;
 	char *icon_uri;
 	char *absolute;
-	char *icon_name;
 
 	icon_uri = g_strdup (gnome_desktop_item_get_string (desktop_file, "X-Nautilus-Icon"));
 	if (icon_uri != NULL) {
@@ -325,19 +319,11 @@ nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop
 	}
 
 	/* Fall back to a standard icon. */
-	icon_name = g_strdup (gnome_desktop_item_get_string (desktop_file, "Icon"));
-	if (icon_name == NULL) {
-		return NULL;
-	}
+	icon_loader = nautilus_icon_factory_get_icon_loader ();
+	absolute = gnome_desktop_item_get_icon (desktop_file, icon_loader);
 	
-	if (icon_name[0] == '/') {
-		absolute = icon_name;
-	} else {
-		absolute = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, icon_name, TRUE, NULL);
-		g_free (icon_name);
-		if (absolute == NULL) {
-			return NULL;
-		}
+	if (absolute == NULL) {
+		return NULL;
 	}
 
 	icon_uri = gnome_vfs_get_uri_from_local_path (absolute);
